library(lme4)
library(coda)
library(runjags)

# set working directory to location of this file
setwd('~')



# MODEL

# y = b0 + b1 * x1 + b2 * x2 + r1.b0[cluster.i] + y.noise
# r1.b0 = N(0, r1.noise)



# PARAMETERS

r1.cluster.n <- 80
n_per_cluster1 <- 10

r1.b0.noise <- 2
r1.b0.mu <- 0

b0 <- 10
b1 <- 0.5
b2 <- 3

y.noise <- 2



# DATA FRAME

n <- r1.cluster.n * n_per_cluster1
r1.cluster.i <- gl(n=r1.cluster.n, k=n_per_cluster1)

x1 <- rnorm(n, 0, 1)
x2 <- rnorm(n, 0, 1)

d <- data.frame(r1.cluster.i = r1.cluster.i, x0 = 1, x1 = x1, x2 = x2, y = NA)



# RANDOM PART

r1.b0 <- c()
for(j in 1:r1.cluster.n) {
	r1.b0 <- c(r1.b0, rnorm(1, r1.b0.mu, r1.b0.noise))
}
r1.b0 <- rep(r1.b0, each = n/r1.cluster.n)



# FIXED PART

for(i in 1:n) {
	
	fixed1 <- b0 + b1 * x1[i] + b2 * x2[i]
	random1 <- r1.b0[i]
	
	d$y[i] <- random1 + fixed1 + rnorm(1,0,y.noise)
}



# LMER MODEL

summary(lmer(y ~ x1 + x2 + (1|r1.cluster.i), data=d))



# JAGS

X <- model.matrix(~ x1 + x2, data=d)
r1.cluster.i <- as.numeric(as.factor(d$r1.cluster.i))

y <- d$y

d_jags <- list (
		n = dim(d)[1], 
		r1.cluster.n = length(unique(r1.cluster.i)), 
		r1.cluster.i = r1.cluster.i,
		y = y,
		X = X,
		P = dim(X)[2]
)


inits1 <- list(
	r1.b0.noise = runif(1, 0, 100),
	y.noise = runif(1, 0, 100),
	b = rnorm(dim(X)[2], 0, 100),
	.RNG.name="base::Super-Duper"
)
inits2 <- list(
	r1.b0.noise = runif(1, 0, 100),
	y.noise = runif(1, 0, 100),
	b = rnorm(dim(X)[2], 0, 100),
	.RNG.name="base::Wichmann-Hill"
)


set.seed(123)
iterations <- 1000 * 2
jags <- run.jags(
		'JAGS/linear 1 - JAGS (one random intercept).R', data = d_jags,
		monitor=c('b', 'r1.b0.mu', 'r1.b0.noise', 'y.noise'), 
		n.chains=2, method='parallel', inits=list(inits1,inits2),     
		sample = iterations,
		burnin = round(iterations/4),
		adapt = round(iterations/10)
)

jags_summary <- print(jags)
rownames(jags_summary)[1:dim(X)[2]] <- colnames(X)
jags_summary