library(lme4)
library(coda)
library(runjags)

# set working directory to location of this file
setwd('~')



# MODEL

# y = b0 + b1 * x1 + b2 * x2 + r1.b0[r1.cluster.i] + r1.b1[r1.cluster.i] * r1.x1[r1.cluster.i] + y.noise
# r1.b0 = N(0, r1.noise)
# r1.b1 = N(r1.b1.mu, r1.noise)



# PARAMETERS

r1.cluster.n <- 40
n_per_cluster1 <- 5

r1.b0.noise <- 1
r1.b0.mu <- 0

r1.b1.noise <- 1
r1.b1.mu <- 4

b0 <- 5
b1 <- 0.5
b2 <- 3

y.noise <- 1



# DATA FRAME

n <- r1.cluster.n * n_per_cluster1
r1.cluster.i <- gl(n=r1.cluster.n, k=n_per_cluster1)
r1.x1 <- rep(1:(n/r1.cluster.n), r1.cluster.n)

x1 <- rnorm(n, 0, 1)
x2 <- rnorm(n, 0, 1)

d <- data.frame(r1.cluster.i = r1.cluster.i, r1.x1 = r1.x1, x0 = 1, x1 = x1, x2 = x2, y = NA)



# RANDOM PART

r1.b0 <- c()
r1.b1 <- c()
for(j in 1:r1.cluster.n) {
	r1.b0 <- c(r1.b0, rnorm(1, r1.b0.mu, r1.b0.noise))
	r1.b1 <- c(r1.b1, rnorm(1, r1.b1.mu, r1.b1.noise))
	# create correlation between r1.b1 and r1.b0
	#r1.b1 <- c(r1.b1, r1.b0[j] + rnorm(1, r1.b1.mu, r1.b1.noise))
	
}
r1.b0 <- rep(r1.b0, each = n/r1.cluster.n)
r1.b1 <- rep(r1.b1, each = n/r1.cluster.n)



# FIXED PART

for(i in 1:n) {
	
	fixed1 <- b0 + b1 * x1[i] + b2 * x2[i]
	random1 <- r1.b0[i] + r1.b1[i] * d$r1.x1[i]
	
	d$y[i] <- random1 + fixed1 + rnorm(1,0,y.noise)
}



# LMER MODEL

summary(lmer(y ~ x1 + x2 + r1.x1 + (1 + r1.x1 | r1.cluster.i), data=d))



# JAGS

X <- model.matrix(~ x1 + x2 + r1.x1, data=d)
r1.cluster.i <- as.numeric(as.factor(d$r1.cluster.i))
r1.x1 <- d$r1.x1

y <- d$y

d_jags <- list (
		n = dim(d)[1], 
		r1.cluster.n = length(unique(r1.cluster.i)), 
		r1.cluster.i = r1.cluster.i,
		r1.x1 = r1.x1,
		y = y,
		X = X,
		P = dim(X)[2],
		r1.p = 2
)


# with such a high range on the uninformative priors (see JAGS file) this model may take a long time to converge
# to speed up convergence for this example, initial values are used that are closer to the true values

inits1 <- list(
	r1.b0.noise = runif(1, 0, 10),
	r1.b1.noise = runif(1, 0, 10),
	r1.rho = runif(1, -1, 1),
	y.noise = runif(1, 0, 10),
	b = rnorm(dim(X)[2], 0, 10),
	.RNG.name="base::Super-Duper"
)
inits2 <- list(
	r1.b0.noise = runif(1, 0, 10),
	r1.b1.noise = runif(1, 0, 10),
	r1.rho = runif(1, -1, 1),
	y.noise = runif(1, 0, 10),
	b = rnorm(dim(X)[2], 0, 10),
	.RNG.name="base::Wichmann-Hill"
)


set.seed(123)
iterations <- 1000 * 50
jags <- run.jags(
		'JAGS/linear 3 - JAGS (one random intercept - one random slope).R', data = d_jags,
		monitor=c('b', 'r1.b0.noise', 'r1.b1.noise', 'r1.rho', 'y.noise'), 
		n.chains=2, method='parallel', inits=list(inits1,inits2),   
		sample = iterations,
		burnin = round(iterations/4),
		adapt = round(iterations/10)
)

jags_summary <- print(jags)
rownames(jags_summary)[1:dim(X)[2]] <- colnames(X)
jags_summary