library(lme4)
library(coda)
library(runjags)
library(MASS)
library(bayesm)

# set working directory to location of this file
setwd('~')



# SIMULATE DATA

# set up observational units

r1.cluster.n <- 50
n_per_cluster <- 10
n <- r1.cluster.n * n_per_cluster
r1.cluster.i <- gl(n=r1.cluster.n, k=n_per_cluster)

P <- 3
Q <- 3


# fixed part

B.fixed <- c(90, 80, 70)

x1.fixed <- rnorm(n, 0, 1); x1.fixed <- (x1.fixed-mean(x1.fixed))/sd(x1.fixed)
x2.fixed <- rnorm(n, 0, 1); x2.fixed <- (x2.fixed-mean(x2.fixed))/sd(x2.fixed)
x3.fixed <- rnorm(n, 0, 1); x3.fixed <- (x3.fixed-mean(x3.fixed))/sd(x3.fixed)

X.fixed <- model.matrix(~ x1.fixed + x2.fixed + x3.fixed -1)

fixed <- X.fixed %*% B.fixed


# random part

B.random <- c(100, 60, 50)

x1.random <- rnorm(n, 0, 1); x1.random <- (x1.random-mean(x1.random))/sd(x1.random)
x2.random <- rnorm(n, 0, 1); x2.random <- (x2.random-mean(x2.random))/sd(x2.random)


rho.B <- matrix(c(
    30,  0.1,  0.3,
   0.1,   50,  0.4,
   0.3,  0.4,   70
), Q, Q)


X.random <- model.matrix(~ r1.cluster.i*x1.random + r1.cluster.i*x2.random -x1.random-x2.random-1)
B.random <- mvrnorm(n=r1.cluster.n, mu=B.random, Sigma=rho.B)

random <- X.random %*% as.vector(B.random)


# construct y

y.hat <- fixed + random

y.noise <- 80
y <- y.hat + rnorm(n, 0, sqrt(y.noise))

d <- data.frame(r1.cluster.i, x1 = x1.fixed, x2 = x2.fixed, x3 = x3.fixed, x4 = x1.random, x5 = x2.random, y)



# ANALYSIS

X.fixed <-  as.matrix(model.frame(~ x1 + x2 + x3 + x4 + x5, data = d))
X.random <- as.matrix(model.frame(~ x4 + x5, data = d))

summary(lmer(y ~ X.fixed + (1 + X.random | r1.cluster.i), data=d))



# JAGS

X.fixed <-  model.matrix(~ x1 + x2 + x3 + x4 + x5, data = d)
X.random <- model.matrix(~ x4 + x5, data = d)

r1.cluster.i <- as.numeric(as.factor(d$r1.cluster.i))

y <- d$y


d_jags <- list (
		n = dim(d)[1], 
		r1.cluster.n = length(unique(r1.cluster.i)), 
		r1.cluster.i = r1.cluster.i,
		X.fixed = X.fixed,
		X.random = X.random,
		P = ncol(X.fixed),
		Q = ncol(X.random),
		y = y,
		W = diag(dim(X.random)[2])
)

inits1 <- list(
	y.noise = runif(1, 0, 100),
	B.fixed = rnorm(ncol(X.fixed), 0, 100),
	xi = runif(ncol(X.random)),
	B.raw = array(rnorm(r1.cluster.n*ncol(X.random)),c(r1.cluster.n,ncol(X.random))),
	Tau.B.raw = rwishart(ncol(X.random)+1,diag(ncol(X.random)))$W, 
	.RNG.name = "base::Super-Duper"
)
inits2 <- list(
	y.noise = runif(1, 0, 100),
	B.fixed = rnorm(ncol(X.fixed), 0, 100),
	xi = runif(ncol(X.random)),
	B.raw = array(rnorm(r1.cluster.n*ncol(X.random)),c(r1.cluster.n,ncol(X.random))),
	Tau.B.raw = rwishart(ncol(X.random)+1,diag(ncol(X.random)))$W, 
	.RNG.name = "base::Super-Duper"
)
inits3 <- list(
	y.noise = runif(1, 0, 100),
	B.fixed = rnorm(ncol(X.fixed), 0, 100),
	xi = runif(ncol(X.random)),
	B.raw = array(rnorm(r1.cluster.n*ncol(X.random)),c(r1.cluster.n,ncol(X.random))),
	Tau.B.raw = rwishart(ncol(X.random)+1,diag(ncol(X.random)))$W, 
	.RNG.name = "base::Super-Duper"
)

set.seed(123)
iterations <- 1000 * 2
jags <- run.jags(
		'JAGS/linear 5 - JAGS (scaled inverse wishart).R', data = d_jags,
		monitor=c('B.fixed', 'sigma.B', 'rho.B', 'y.noise'), 
		n.chains=2, method='parallel', inits=list(inits1,inits2),   
		sample = iterations,
		burnin = round(iterations/4),
		adapt = round(iterations/10)
)

jags_summary <- print(jags)
rownames(jags_summary)[grepl('B.fixed', rownames(jags_summary))]  <- colnames(X.fixed)
rownames(jags_summary)[grepl('sigma.B', rownames(jags_summary))] <- paste('SD', colnames(X.random), sep='_')

jags_summary

